@@ -116,7 +116,7 @@ class Index:
     def search(self, q: Union[str, dict], searchable_attributes: Optional[List[str]] = None,
                limit: int = 10, offset: int = 0, search_method: Union[SearchMethods.TENSOR, str] = SearchMethods.TENSOR,
                highlights=None, device: Optional[str] = None, filter_string: str = None,
-               show_highlights=True, reranker=None,
+               show_highlights=True, reranker=None, image_download_headers: Optional[Dict] = None,
                attributes_to_retrieve: Optional[List[str]] = None, boost: Optional[Dict[str,List[Union[float, int]]]] = None,
                ) -> Dict[str, Any]:
         """Search the index.
@@ -126,9 +126,9 @@ class Index:
                 (with the structure <search string>:<weight float>). Strings
                 to search are text or a pointer/url to an image if the index
                 has treat_urls_and_pointers_as_images set to True. 
-                
-                If queries are weighted, each weight act as a (possibly negative) 
-                multiplier for that query, relative to the other queries. 
+
+                If queries are weighted, each weight act as a (possibly negative)
+                multiplier for that query, relative to the other queries.
             searchable_attributes:  attributes to search
             limit: The max number of documents to be returned
             offset: The number of search results to skip (for pagination)
@@ -172,20 +172,23 @@ class Index:
             body["attributesToRetrieve"] = attributes_to_retrieve
         if filter_string is not None:
             body["filter"] = filter_string
+        if image_download_headers is not None:
+            body["image_download_headers"] = image_download_headers
         res = self.http.post(
             path=path_with_query_str,
             body=body
         )
-        
+
         num_results = len(res["hits"])
         end_time_client_request = timer()
         total_client_request_time = end_time_client_request - start_time_client_request
-        
-        search_time_log = f"search ({search_method.lower()}): took {(total_client_request_time):.3f}s to send query and received {num_results} results from Marqo (roundtrip)."
+
+        search_time_log = (f"search ({search_method.lower()}): took {(total_client_request_time):.3f}s to send query "
+                           f"and received {num_results} results from Marqo (roundtrip).")
         if 'processingTimeMs' in res:
             search_time_log += f" Marqo itself took {(res['processingTimeMs'] * 0.001):.3f}s to execute the search."
-        
-        mq_logger.info(search_time_log)
+
+        mq_logger.debug(search_time_log)
         return res
 
     def get_document(self, document_id: str, expose_facets=None) -> Dict[str, Any]:
@@ -233,7 +236,9 @@ class Index:
         client_batch_size: int = None,
         processes: int = None,
         device: str = None,
-        non_tensor_fields: List[str] = None
+        non_tensor_fields: List[str] = None,
+        use_existing_tensors: bool = False,
+        image_download_headers: dict = None
     ) -> Union[Dict[str, Any], List[Dict[str, Any]]]:
         """Add documents to this index. Does a partial update on existing documents,
         based on their ID. Adds unseen documents to the index.
@@ -253,16 +258,22 @@ class Index:
             device: the device used to index the data. Examples include "cpu",
                 "cuda" and "cuda:2"
             non_tensor_fields: fields within documents to not create and store tensors against.
+            use_existing_tensors: use vectors that already exist in the docs.
+            image_download_headers: a dictionary of headers to be passed while downloading images,
+                for URLs found in documents
 
         Returns:
             Response body outlining indexing result
         """
         if non_tensor_fields is None:
             non_tensor_fields = []
+        if image_download_headers is None:
+            image_download_headers = dict()
         return self._generic_add_update_docs(
             update_method="replace",
             documents=documents, auto_refresh=auto_refresh, server_batch_size=server_batch_size,
-            client_batch_size=client_batch_size, processes=processes, device=device, non_tensor_fields=non_tensor_fields
+            client_batch_size=client_batch_size, processes=processes, device=device, non_tensor_fields=non_tensor_fields,
+            use_existing_tensors=use_existing_tensors, image_download_headers=image_download_headers
         )
 
     def update_documents(
@@ -273,9 +284,11 @@ class Index:
         client_batch_size: int = None,
         processes: int = None,
         device: str = None,
-        non_tensor_fields: List[str] = None
+        non_tensor_fields: List[str] = None,
     ) -> Union[Dict[str, Any], List[Dict[str, Any]]]:
-        """Add documents to this index. Does a partial updates on existing documents,
+        """
+        Will be deprecated soon.
+        Add documents to this index. Does a partial updates on existing documents,
         based on their ID. Adds unseen documents to the index.
 
         Args:
@@ -302,7 +315,7 @@ class Index:
         return self._generic_add_update_docs(
             update_method="update",
             documents=documents, auto_refresh=auto_refresh, server_batch_size=server_batch_size,
-            client_batch_size=client_batch_size, processes=processes, device=device, non_tensor_fields=non_tensor_fields
+            client_batch_size=client_batch_size, processes=processes, device=device, non_tensor_fields=non_tensor_fields,
         )
 
     def _generic_add_update_docs(
@@ -314,8 +327,14 @@ class Index:
         client_batch_size: int = None,
         processes: int = None,
         device: str = None,
-        non_tensor_fields: List[str] = []
+        non_tensor_fields: List = None,
+        use_existing_tensors: bool = False,
+        image_download_headers: dict = None
     ) -> Union[Dict[str, Any], List[Dict[str, Any]]]:
+
+        if non_tensor_fields is None:
+            non_tensor_fields = []
+
         selected_device = device if device is not None else self.config.indexing_device
         num_docs = len(documents)
 
@@ -324,15 +343,20 @@ class Index:
         start_time_client_process = timer()
         base_path = f"indexes/{self.index_name}/documents"
         non_tensor_fields_query_param = utils.convert_list_to_query_params("non_tensor_fields", non_tensor_fields)
+        image_download_headers_param = (utils.convert_dict_to_url_params(image_download_headers)
+                                        if image_download_headers else '')
         query_str_params = (
             f"{f'&device={utils.translate_device_string_for_url(selected_device)}'}"
             f"{f'&processes={processes}' if processes is not None else ''}"
             f"{f'&batch_size={server_batch_size}' if server_batch_size is not None else ''}"
+            f"{f'&use_existing_tensors={str(use_existing_tensors).lower()}' if use_existing_tensors is not None else ''}"
             f"{f'&{non_tensor_fields_query_param}' if len(non_tensor_fields) > 0 else ''}"
+            f"{f'&image_download_headers={image_download_headers_param}' if image_download_headers else ''}"
         )
         end_time_client_process = timer()
         total_client_process_time = end_time_client_process - start_time_client_process
-        mq_logger.info(f"add_documents pre-processing: took {(total_client_process_time):.3f}s for {num_docs} docs, for an average of {(total_client_process_time / num_docs):.3f}s per doc.")
+        mq_logger.debug(f"add_documents pre-processing: took {(total_client_process_time):.3f}s for {num_docs} docs, "
+                       f"for an average of {(total_client_process_time / num_docs):.3f}s per doc.")
 
         if client_batch_size is not None:
             if client_batch_size <= 0:
@@ -342,7 +366,7 @@ class Index:
                 update_method=update_method, docs=documents, verbose=False,
                 query_str_params=query_str_params, batch_size=client_batch_size
             )
-            
+
         else:
             # no Client Batching
             refresh_option = f"?refresh={str(auto_refresh).lower()}"
@@ -358,43 +382,43 @@ class Index:
             else:
                 raise ValueError(f'Received unknown update_method `{update_method}`. '
                                  f'Allowed update_methods: ["replace", "update"] ')
-            
-            
+
+
             end_time_client_request = timer()
             total_client_request_time = end_time_client_request - start_time_client_request
-            
-            mq_logger.info(f"add_documents roundtrip: took {(total_client_request_time):.3f}s to send {num_docs} "
+
+            mq_logger.debug(f"add_documents roundtrip: took {(total_client_request_time):.3f}s to send {num_docs} "
                             f"docs to Marqo (roundtrip, unbatched), for an average of {(total_client_request_time / num_docs):.3f}s per doc.")
 
             if server_batch_size is not None:
                 # with Server Batching (show processing time for each batch)
-                mq_logger.info(f"add_documents Marqo index (server-side batch reports): ")
+                mq_logger.debug(f"add_documents Marqo index (server-side batch reports): ")
 
                 if processes is not None and processes > 1:
                     # for multiprocess, timing messages should be arranged by process, then batch
                     for process in range(len(res)):
-                        mq_logger.info(f"   process {process}:")
+                        mq_logger.debug(f"   process {process}:")
 
                         for batch in range(len(res[process])):
                             server_batch_result_count = len(res[process][batch]["items"])
-                            mq_logger.info(f"       marqo server batch {batch}: "
+                            mq_logger.debug(f"       marqo server batch {batch}: "
                                             f"processed {server_batch_result_count} docs in {(res[process][batch]['processingTimeMs'] / 1000):.3f}s, "
                                             f"for an average of {(res[process][batch]['processingTimeMs'] / (1000 * server_batch_result_count)):.3f}s per doc.")
                 else:
                     # for single process, timing messages should be arranged by batch ONLY
                     for batch in range(len(res)):
                         server_batch_result_count = len(res[batch]["items"])
-                        mq_logger.info(f"   marqo server batch {batch}: "
+                        mq_logger.debug(f"   marqo server batch {batch}: "
                                         f"processed {server_batch_result_count} docs in {(res[batch]['processingTimeMs'] / 1000):.3f}s, "
                                         f"for an average of {(res[batch]['processingTimeMs'] / (1000 * server_batch_result_count)):.3f}s per doc.")
             else:
                 # no Server Batching
                 if 'processingTimeMs' in res:       # Only outputs log if response is non-empty
-                    mq_logger.info(f"add_documents Marqo index: took {(res['processingTimeMs'] / 1000):.3f}s for Marqo to process & index {num_docs} "
+                    mq_logger.debug(f"add_documents Marqo index: took {(res['processingTimeMs'] / 1000):.3f}s for Marqo to process & index {num_docs} "
                                     f"docs (server unbatched), for an average of {(res['processingTimeMs'] / (1000 * num_docs)):.3f}s per doc.")
 
         total_add_docs_time = timer() - t0
-        mq_logger.info(f"add_documents completed. total time taken: {(total_add_docs_time):.3f}s.")
+        mq_logger.debug(f"add_documents completed. total time taken: {(total_add_docs_time):.3f}s.")
         return res
 
     def delete_documents(self, ids: List[str], auto_refresh: bool = None) -> Dict[str, int]:
@@ -452,7 +476,7 @@ class Index:
         """
         path_with_query_str = f"{base_path}?refresh=false{query_str_params}"
 
-        mq_logger.info(f"starting batch ingestion with batch size {batch_size}")
+        mq_logger.debug(f"starting batch ingestion with batch size {batch_size}")
 
         deeper = ((doc, i, batch_size) for i, doc in enumerate(docs))
 
@@ -477,35 +501,37 @@ class Index:
                                  f'Allowed update_methods: ["replace", "update"] ')
             total_batch_time = timer() - t0
             num_docs = len(docs)
-            mq_logger.info(
-                f"   add_documents batch {i} roundtrip: took {(total_batch_time):.3f}s to add {num_docs} docs, "
-                f"for an average of {(total_batch_time/num_docs):.3f}s per doc.")
 
             if isinstance(res, list):
                 # with Server Batching (show processing time for each batch)
+                mq_logger.info(
+                    f"   add_documents batch {i} roundtrip: took {(total_batch_time):.3f}s to add {num_docs} docs, "
+                    f"for an average of {(total_batch_time / num_docs):.3f}s per doc.")
 
                 if isinstance(res[0], list):
                     # for multiprocess, timing messages should be arranged by process, then batch
                     for process in range(len(res)):
-                        mq_logger.info(f"       process {process}:")
+                        mq_logger.debug(f"       process {process}:")
 
                         for batch in range(len(res[process])):
                             server_batch_result_count = len(res[process][batch]["items"])
-                            mq_logger.info(f"           marqo server batch {batch}: "
+                            mq_logger.debug(f"           marqo server batch {batch}: "
                                             f"processed {server_batch_result_count} docs in {(res[process][batch]['processingTimeMs'] / 1000):.3f}s, "
                                             f"for an average of {(res[process][batch]['processingTimeMs'] / (1000 * server_batch_result_count)):.3f}s per doc.")
                 else:
                     # for single process, timing messages should be arranged by batch ONLY
                     for batch in range(len(res)):
                         server_batch_result_count = len(res[batch]["items"])
-                        mq_logger.info(f"       marqo server batch {batch}: "
+                        mq_logger.debug(f"       marqo server batch {batch}: "
                                         f"processed {server_batch_result_count} docs in {(res[batch]['processingTimeMs'] / 1000):.3f}s, "
                                         f"for an average of {(res[batch]['processingTimeMs'] / (1000 * server_batch_result_count)):.3f}s per doc.")
             else:
                 # no Server Batching
                 if 'processingTimeMs' in res:       # Only outputs log if response is non-empty
-                    mq_logger.info(f"   add_documents batch {i} Marqo processing: took {(res['processingTimeMs'] / 1000):.3f}s for Marqo to process & index {num_docs} "
-                                    f"docs (server unbatched), for an average of {(res['processingTimeMs'] / (1000 * num_docs)):.3f}s per doc.")
+                    mq_logger.info(
+                        f"   add_documents batch {i}: took {(res['processingTimeMs'] / 1000):.3f}s for Marqo to process & index {num_docs} "
+                        f"docs (server unbatched), for an average of {(res['processingTimeMs'] / (1000 * num_docs)):.3f}s per doc."
+                        f" Roundtrip time: {(total_batch_time):.3f}s")
 
 
             if verbose:
@@ -515,7 +541,7 @@ class Index:
         results = [verbosely_add_docs(i, docs) for i, docs in enumerate(batched)]
         if auto_refresh:
             self.refresh()
-        mq_logger.info('completed batch ingestion.')
+        mq_logger.debug('completed batch ingestion.')
         return results
 
     def get_settings(self) -> dict:
